import setuptools
from setuptools import setup
import os


# Read the content of `README.md`:
def read_readme():
    with open(os.path.join(os.getcwd(), 'README.md'), 'r', encoding='utf-8') as file:
        return file.read()


setup(
    name="TitanDevice",
    version="0.2.6",
    author="369",
    author_email="luck.yangbo@gmai.com",
    description="A Python library used for managing device",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'adbutils',
        'pydantic'
    ],
)



class DeviceNoFoundException(Exception):
    def __init__(self, device_serial):
        super().__init__(f"Device with serial {device_serial} not found")


class DeviceMustBeRootedException(Exception):
    def __init__(self, device_serial):
        super().__init__(
            f'Device [{device_serial}] must be rooted to perform this operation'
        )


class FridaServerNotInstalled(Exception):
    def __init__(self, device_serial, frida_server_path):
        super().__init__(
            f'Frida server is not installed in {frida_server_path} on device with serial {device_serial}'
        )



from titan._device_exception import *
from titan._device_manager import _DeviceManager
from titan._device_models import DeviceInfo, FridaInfo

device_manager_list: list[_DeviceManager] = _DeviceManager.get_all_device_manager_list()


def get_all_devices() -> list[DeviceInfo]:
    global device_manager_list
    device_manager_list = _DeviceManager.get_all_device_manager_list()
    return [device.get_info() for device in device_manager_list]


def get_frida_info(
        device_serial: str,
        frida_server_name='frida-server',
        frida_server_path='/data/local/tmp/frida-server'
) -> FridaInfo:
    for device in device_manager_list:
        if device.get_info().serial == device_serial:
            return device.get_frida_info(frida_server_path, frida_server_name)
    raise DeviceNoFoundException(device_serial)


def install_frida(
        device_serial: str, frida_server_path: str,
        dist_path='/data/local/tmp/frida-server'
) -> FridaInfo:
    for device in device_manager_list:
        if device.get_info().serial == device_serial:
            return device.install_frida(frida_server_path, dist_path)
    raise DeviceNoFoundException(device_serial)


def start_frida(
        device_serial: str, frida_server_path='/data/local/tmp/frida-server'
) -> FridaInfo:
    for device in device_manager_list:
        if device.get_info().serial == device_serial:
            return device.start_frida(frida_server_path)
    raise DeviceNoFoundException(device_serial)


def stop_frida(
        device_serial: str, frida_server_path='/data/local/tmp/frida-server'
) -> FridaInfo:
    for device in device_manager_list:
        if device.get_info().serial == device_serial:
            return device.stop_frida(frida_server_path)
    raise DeviceNoFoundException(device_serial)


def uninstall_frida(
        device_serial: str, frida_server_path='/data/local/tmp/frida-server'
) -> FridaInfo:
    for device in device_manager_list:
        if device.get_info().serial == device_serial:
            return device.uninstall_frida(frida_server_path)
    raise DeviceNoFoundException(device_serial)
